@@ -1,3 +1,3 @@
-from pegtree.pasm import ParseTree
+from .tree import ParseTree
 from pegtree.loader import grammar
 from pegtree.nez import generate
