@@ -1,24 +1,43 @@
 from collections import namedtuple
+from .tree import ParseTree
+
+# PContext
 
 
-def pRule(peg, name, pf):
-    peg[name] = pf
+class PContext:
+    __slots__ = ['inputs', 'pos', 'epos',
+                 'headpos', 'ptree', 'state', 'memo', 'dic']
 
-# Generator
+    def __init__(self, inputs, spos, epos):
+        self.inputs = inputs
+        self.pos = spos
+        self.epos = epos
+        self.headpos = spos
+        self.ptree = None
+        self.state = None
+        self.memo = [PMemo() for x in range(1789)]
+        self.dic = {}
 
 
-def match_empty(px): return True
+# '' pEmpty()
+
+def match_empty(px: PContext):
+    return True
 
 
 def pEmpty():
     return match_empty
 
 
+# !'' pFail()
+
 def pFail():
     return lambda px: False
 
+# . pAny()
 
-def match_any(px):
+
+def match_any(px: PContext):
     if px.pos < px.epos:
         px.pos += 1
         return True
@@ -28,89 +47,33 @@ def match_any(px):
 def pAny():
     return match_any
 
+# a pChar(a)
 
-CharCache = {
+
+CharDB = {
     '': match_empty
 }
 
 
 def pChar(text):
-    if text in CharCache:
-        return CharCache[text]
+    if text in CharDB:
+        return CharDB[text]
+
     clen = len(text)
 
-    def match_char(px):
+    def match(px: PContext):
         if px.inputs.startswith(text, px.pos):
             px.pos += clen
             return True
         return False
-    CharCache[text] = match_char
-    return match_char
-
-# Range
+    CharDB[text] = match
+    return match
 
 
-BitmapCache = {}
+# [abcA-Z] pRange('abc', 'A-Z')
 
 
-def unique_range(chars, ranges, memo=None):
-    cs = 0
-    for c in chars:
-        cs |= 1 << ord(c)
-    r = ranges
-    while len(r) > 1:
-        for c in range(ord(r[0]), ord(r[1])+1):
-            cs |= 1 << c
-        r = r[2:]
-    if memo is not None:
-        if cs in memo:
-            return memo[cs]
-        memo[cs] = cs
-    return cs
-
-
-'''
-def minimum_range(chars, ranges):
-    cs = 0xffff
-    for c in chars:
-        cs = min(cs, ord(c))
-    r = ranges
-    while len(r) > 1:
-        cs = min(cs, ord(r[0]))
-        cs = min(cs, ord(r[1]))
-        r = r[2:]
-    return cs
-
-
-
-
-def bitmap(chars, ranges):
-    key = (chars, ranges)
-    if key in BitmapCache:
-        return BitmapCache[key]
-    offset = minimum_range(chars, ranges)
-    bitset = unique_range(chars, ranges) >> offset
-    BitmapCache[key] = (bitset, offset)
-    return BitmapCache[key]
-
-
-def pRange(chars, ranges):
-    bitset, offset = bitmap(chars, ranges)
-
-    def match_bitset(px):
-        if px.pos < px.epos:
-            shift = ord(px.inputs[px.pos]) - offset
-            if shift >= 0 and (bitset & (1 << shift)) != 0:
-                px.pos += 1
-                return True
-        return False
-    return match_bitset
-'''
-
-# NewRange
-
-
-def tochars(chars, ranges):
+def make_allchars(chars, ranges):
     cs = set(list(chars))
     rs = ranges
     while len(rs) > 0:
@@ -137,40 +100,42 @@ def make_bitset(chars, ranges, NBITS=8):
     return bitmap, cmin, len(bitmap)*NBITS
 
 
-def range_pattern(chars, ranges):
-    key = tochars(chars, ranges)
-    if len(key) < 1000:
-        return key
-    if key in BitmapCache:
-        return BitmapCache[key]
-    BitmapCache[key] = make_bitset(chars, ranges)
-    return BitmapCache[key]
+# def match_chars(px, chars):
+#     return px.pos < px.epos and chars.find(px.inputs[px.pos]) != -1
+
+# def match_bitset(px, bitset, offset, maxlen):
+#     if px.pos < px.epos:
+#         c = ord(px.inputs[px.pos]) - offset
+#         if c < 0 or c >= maxlen:
+#             return False
+#         mask = 1 << c % 8
+#         return bitset[c//8] & mask == mask
+#     return False
+
+BitsetDB = {}
 
 
-def match_chars(px, chars):
-    return px.pos < px.epos and chars.find(px.inputs[px.pos]) != -1
+def check_range(chars, ranges=''):
+    if isinstance(chars, tuple):
+        return chars[0], chars[1]
+    return chars, ranges
 
 
-def match_bitset(px, bitset, offset, maxlen):
-    if px.pos < px.epos:
-        c = ord(px.inputs[px.pos]) - offset
-        if c < 0 or c >= maxlen:
-            return False
-        mask = 1 << c % 8
-        return bitset[c//8] & mask == mask
-    return False
+def pAndRange(chars, ranges=''):
+    chars, ranges = check_range(chars, ranges)
+    allchars = make_allchars(chars, ranges)
 
+    if len(allchars) < 1000:
+        def match(px):
+            return px.pos < px.epos and allchars.find(px.inputs[px.pos]) != -1
+        return match
 
-def pAndRange(chars, ranges):
-    pat = range_pattern(chars, ranges)
-    if isinstance(pat, str):
-        def match_chars(px):
-            return px.pos < px.epos and pat.find(px.inputs[px.pos]) != -1
-        return match_chars
+    if allchars not in BitsetDB:
+        BitsetDB[allchars] = make_bitset(chars, ranges)
 
-    bitset, offset, maxlen = pat
+    bitset, offset, maxlen = BitsetDB[allchars]
 
-    def match_ranges(px):
+    def match(px: PContext):
         if px.pos < px.epos:
             c = ord(px.inputs[px.pos]) - offset
             if c < 0 or c >= maxlen:
@@ -178,64 +143,136 @@ def pAndRange(chars, ranges):
             mask = 1 << (c % 8)
             return bitset[c//8] & mask == mask
         return False
-    return match_ranges
+    return match
 
 
-def pRange(chars, ranges):
+def pRange(chars, ranges=''):
     pf = pAndRange(chars, ranges)
 
-    def match_ranges(px):
+    def match(px: PContext):
         if pf(px):
             px.pos += 1
             return True
         return False
-    return match_ranges
+    return match
+
+# &e, pAnd(e)
 
 
-def pAnd(pf):
-    def match_and(px):
+def pAnd_(e):
+    def match(px: PContext):
         pos = px.pos
-        if pf(px):
+        if e(px):
             px.headpos = max(px.pos, px.headpos)
             px.pos = pos
             return True
         return False
-    return match_and
+    return match
 
 
-def pNot(pf):
-    def match_not(px):
+def pAndChar(text):
+    def match(px: PContext):
+        return px.inputs.startswith(text, px.pos)
+    return match
+
+
+def pAnd(e):
+    if isinstance(e, str):
+        return pAndChar(e)
+    elif isinstance(e, tuple):
+        return pAndRange(e)
+    else:
+        return pAnd_(e)
+
+# !e pNot(e)
+
+
+def pNot_(e):
+    def match(px: PContext):
         pos = px.pos
         ptree = px.ptree
-        if not pf(px):
+        if not e(px):
             px.headpos = max(px.pos, px.headpos)
             px.pos = pos
             px.ptree = ptree
             return True
         return False
-    return match_not
+    return match
 
 
-def pMany(pf):
-    def match_many(px):
+def pNotChar(text):
+    def match_notchar(px: PContext):
+        return not px.inputs.startswith(text, px.pos)
+    return match_notchar
+
+
+def pNotRange(chars, ranges=''):
+    e = pAndRange(chars, ranges)
+    return lambda px: not e(px)
+
+
+def pNot(e):
+    if isinstance(e, str):
+        return pNotChar(e)
+    elif isinstance(e, tuple):
+        return pNotRange(e)
+    else:
+        return pNot_(e)
+
+
+# e* pMany(e)
+
+def pMany_(e):
+    def match(px: PContext):
         pos = px.pos
         ptree = px.ptree
-        while pf(px) and pos < px.pos:
+        while e(px) and pos < px.pos:
             pos = px.pos
             ptree = px.ptree
         px.headpos = max(px.pos, px.headpos)
         px.pos = pos
         px.ptree = ptree
         return True
-    return match_many
+    return match
 
 
-def pOneMany(pf):
-    def match_OneMany(px):
-        if pf(px):
+def pManyChar(text):
+    clen = len(text)
+
+    def match(px: PContext):
+        while px.inputs.startswith(text, px.pos):
+            px.pos += clen
+        return True
+    return match
+
+
+def pManyRange(chars, ranges=''):
+    e = pAndRange(chars, ranges)
+
+    def match(px: PContext):
+        while e(px):
+            px.pos += 1
+        return True
+    return match
+
+
+def pMany(e):
+    if isinstance(e, str):
+        return pManyChar(e)
+    elif isinstance(e, tuple):
+        return pManyRange(e)
+    else:
+        return pMany_(e)
+
+# e! pOneMany(e)
+
+
+def pOneMany_(e):
+    def match(px: PContext):
+        if e(px):
             pos = px.pos
             ptree = px.ptree
-            while pf(px) and pos < px.pos:
+            while e(px) and pos < px.pos:
                 pos = px.pos
                 ptree = px.ptree
             px.headpos = max(px.pos, px.headpos)
@@ -243,118 +280,185 @@ def pOneMany(pf):
             px.ptree = ptree
             return True
         return False
-    return match_OneMany
+    return match
 
 
-def pOption(pf):
-    def match_option(px):
+def pOneManyChar(text):
+    clen = len(text)
+
+    def match(px: PContext):
+        if px.inputs.startswith(text, px.pos):
+            px.pos += clen
+            while px.inputs.startswith(text, px.pos):
+                px.pos += clen
+            return True
+        return False
+    return match
+
+
+def pOneManyRange(chars, ranges=''):
+    e = pAndRange(chars, ranges)
+
+    def match(px: PContext):
+        c = 0
+        while e(px):
+            px.pos += 1
+            c += 1
+        return c > 0
+    return match
+
+
+def pOneMany(e):
+    if isinstance(e, str):
+        return pOneManyChar(e)
+    elif isinstance(e, tuple):
+        return pOneManyRange(e)
+    else:
+        return pOneMany_(e)
+
+
+# e? pOption(e)
+
+def pOption_(e):
+    def match(px: PContext):
         pos = px.pos
         ptree = px.ptree
-        if not pf(px):
+        if not e(px):
             px.headpos = max(px.pos, px.headpos)
             px.pos = pos
             px.ptree = ptree
         return True
-    return match_option
+    return match
+
+
+def pOptionChar(text):
+    clen = len(text)
+
+    def match(px: PContext):
+        if px.inputs.startswith(text, px.pos):
+            px.pos += clen
+        return True
+    return match
+
+
+def pOptionRange(chars, ranges):
+    e = pAndRange(chars, ranges)
+
+    def match(px: PContext):
+        if e(px):
+            px.pos += 1
+        return True
+    return match
+
+
+def pOption(e):
+    if isinstance(e, str):
+        return pOptionChar(e)
+    elif isinstance(e, tuple):
+        return pOptionRange(e)
+    else:
+        return pOption_(e)
+
 
 # Seq
 
-
-def pSeq2(pf, pf2):
-    def match_seq2(px):
-        return pf(px) and pf2(px)
-    return match_seq2
+def pSeq2(e, e2):
+    def match(px: PContext):
+        return e(px) and e2(px)
+    return match
 
 
-def pSeq3(pf, pf2, pf3):
-    def match_seq3(px):
-        return pf(px) and pf2(px) and pf3(px)
-    return match_seq3
+def pSeq3(e, e2, e3):
+    def match(px: PContext):
+        return e(px) and e2(px) and e3(px)
+    return match
 
 
-def pSeq4(pf, pf2, pf3, pf4):
-    def match_seq4(px):
-        return pf(px) and pf2(px) and pf3(px) and pf4(px)
-    return match_seq4
+def pSeq4(e, e2, e3, e4):
+    def match(px: PContext):
+        return e(px) and e2(px) and e3(px) and e4(px)
+    return match
 
 
-def pSeq(*pfs):
-    def match_seq(px):
-        for pf in pfs:
-            if not pf(px):
+def pSeq(*es):
+    def match(px: PContext):
+        for e in es:
+            if not e(px):
                 return False
         return True
-    return match_seq
+    return match
 
 # Ore
 
 
-def pOre2(pf, pf2):
-    def match_ore2(px):
+def pOre2(e, e2):
+    def match(px: PContext):
         pos = px.pos
         ptree = px.ptree
-        if pf(px):
+        if e(px):
             return True
         px.headpos = max(px.pos, px.headpos)
         px.pos = pos
         px.ptree = ptree
-        return pf2(px)
-    return match_ore2
+        return e2(px)
+    return match
 
 
-def pOre3(pf, pf2, pf3):
-    def match_ore3(px):
+def pOre3(e, e2, e3):
+    def match(px: PContext):
         pos = px.pos
         ptree = px.ptree
-        if pf(px):
+        if e(px):
             return True
         px.headpos = max(px.pos, px.headpos)
         px.pos = pos
         px.ptree = ptree
-        if pf2(px):
+        if e2(px):
             return True
         px.headpos = max(px.pos, px.headpos)
         px.pos = pos
         px.ptree = ptree
-        return pf3(px)
-    return match_ore3
+        return e3(px)
+    return match
 
 
-def pOre4(pf, pf2, pf3, pf4):
-    def match_ore4(px):
+def pOre4(e, e2, e3, e4):
+    def match(px: PContext):
         pos = px.pos
         ptree = px.ptree
-        if pf(px):
+        if e(px):
             return True
         px.headpos = max(px.pos, px.headpos)
         px.pos = pos
         px.ptree = ptree
-        if pf2(px):
+        if e2(px):
             return True
         px.headpos = max(px.pos, px.headpos)
         px.pos = pos
         px.ptree = ptree
-        if pf3(px):
+        if e3(px):
             return True
         px.headpos = max(px.pos, px.headpos)
         px.pos = pos
         px.ptree = ptree
-        return pf4(px)
-    return match_ore4
+        return e4(px)
+    return match
 
 
-def pOre(*pfs):
-    def match_ore(px):
+def pOre(*es):
+    def match(px: PContext):
         pos = px.pos
         ptree = px.ptree
-        for pf in pfs:
-            if pf(px):
+        for e in es:
+            if e(px):
                 return True
             px.headpos = max(px.pos, px.headpos)
             px.pos = pos
             px.ptree = ptree
         return False
-    return match_ore
+    return match
+
+# pDict('a b c')
 
 
 def make_trie(dic):
@@ -435,7 +539,7 @@ def pMemo(fs, mp, mpsize):
     hit = 0
     miss = 0
 
-    def match_memo(px):
+    def match(px: PContext):
         nonlocal disabled, hit, miss
         if disabled:
             return fs(px)
@@ -467,7 +571,7 @@ def pMemo(fs, mp, mpsize):
             if hit / miss < 5:
                 disabled = True
         return m.result
-    return match_memo
+    return match
 
 
 def pMemoDebug(name, fs, mp, mps):
@@ -476,7 +580,7 @@ def pMemoDebug(name, fs, mp, mps):
     miss = 0
     mpsize = len(mps)
 
-    def match_memo(px):
+    def match(px: PContext):
         nonlocal disabled, hit, miss
         if disabled:
             return fs(px)
@@ -510,7 +614,7 @@ def pMemoDebug(name, fs, mp, mps):
                 print('enabled', mps)
                 disabled = True
         return m.result
-    return match_memo
+    return match
 
 
 # Tree Construction
@@ -543,30 +647,30 @@ class PTree(object):
         return ''.join(sb)
 
 
-def pNode(pf, tag, shift):
-    def make_tree(px):
+def pNode(e, tag, shift):
+    def make(px: PContext):
         pos = px.pos
         prev = px.ptree
         px.ptree = None
-        if pf(px):
+        if e(px):
             px.ptree = PTree(prev, tag, pos+shift, px.pos, px.ptree)
             return True
         return False
-    return make_tree
+    return make
 
 
-def pEdge(edge, pf, shift=0):
-    def match_edge(px):
+def pEdge(edge, e, shift=0):
+    def match(px: PContext):
         pos = px.pos
         prev = px.ptree
         px.ptree = None
-        if pf(px):
+        if e(px):
             if px.ptree is None:
                 px.ptree = PTree(None, '', pos+shift, px.pos, px.ptree)
             px.ptree = PTree(prev, edge, -1, -1, px.ptree)
             return True
         return False
-    return match_edge
+    return match
 
 
 def popPTree(px):
@@ -580,42 +684,43 @@ def popPTree(px):
     return pt.spos, pt.prev
 
 
-def pFold(edge, pf, tag, shift):
+def pFold(edge, e, tag, shift):
     if edge == '':
-        def match_fold(px):
+        def match(px: PContext):
             pos, prev = popPTree(px)
-            if pf(px):
+            if e(px):
                 px.ptree = PTree(prev, tag, pos, px.pos, px.ptree)
                 return True
             return False
-        return match_fold
+        return match
     else:
-        def match_fold2(px):
+        def match(px: PContext):
             pos, prev = popPTree(px)
             px.ptree = PTree(None, edge, -1, -1, px.ptree)
-            if pf(px):
+            if e(px):
                 px.ptree = PTree(prev, tag, pos, px.pos, px.ptree)
                 return True
             return False
-        return match_fold2
+        return match
 
 
-def pAbs(pf):
-    def match_abs(px):
+def pAbs(e):
+    def match(px: PContext):
         ptree = px.ptree
-        if pf(px):
+        if e(px):
             px.ptree = ptree
             return True
         return False
-    return match_abs
+    return match
 
 
 def pSkip():  # @skip()
-    def skip(px):
+    def skip(px: PContext):
         px.pos = min(px.headpos, px.epos)
         return True
     return skip
 
+
 # State
 
 
@@ -630,23 +735,23 @@ def getstate(state, sid):
     return None
 
 
-def pSymbol(pf, sid):  # @symbol(A)
-    def match_symbol(px):
+def pSymbol(e, sid):  # @symbol(A)
+    def match(px: PContext):
         pos = px.pos
-        if pf(px):
+        if e(px):
             px.state = State(sid, px.inputs[pos:px.pos], px.state)
             return True
         return False
-    return match_symbol
+    return match
 
 
-def pScope(pf):
-    def scope(px):
+def pScope(e):
+    def match(px: PContext):
         state = px.state
-        res = pf(px)
+        res = e(px)
         px.state = state
         return res
-    return scope
+    return match
 
 
 def pExists(sid):  # @Match(A)
@@ -654,7 +759,7 @@ def pExists(sid):  # @Match(A)
 
 
 def pMatch(sid):  # @Match(A)
-    def match(px):
+    def match(px: PContext):
         state = getstate(px.state, sid)
         if state is not None and px.inputs.startswith(state.val, px.pos):
             px.pos += len(state.val)
@@ -662,15 +767,11 @@ def pMatch(sid):  # @Match(A)
         return False
     return match
 
-# params = pe.params
-# name = str(params[0])
-# pf = self.emit(pe.e, step)
 
-
-def pDef(name, pf):
-    def define_dic(px):
+def pDef(name, e):
+    def define_dic(px: PContext):
         pos = px.pos
-        if pf(px):
+        if e(px):
             s = px.inputs[pos:px.pos]
             if len(s) == 0:
                 return True
@@ -685,11 +786,8 @@ def pDef(name, pf):
     return define_dic
 
 
-# params = pe.params
-# name = str(params[0])
-
 def pIn(name):  # @in(NAME)
-    def match_dic(px):
+    def match(px: PContext):
         # print('@matching', name, px.inputs, px.pos)
         if name in px.dic:
             ss = px.dic[name]
@@ -700,288 +798,11 @@ def pIn(name):  # @in(NAME)
                     return True
             # print('@', px.inputs[px.pos:], ss)
         return False
-    return match_dic
-
-
-# Optimized
-
-
-def pAndChar(text):
-    def match_andchar(px):
-        return px.inputs.startswith(text, px.pos)
-    return match_andchar
-
-
-def pNotChar(text):
-    def match_notchar(px):
-        return not px.inputs.startswith(text, px.pos)
-    return match_notchar
-
-
-def pManyChar(text):
-    clen = len(text)
-
-    def match_manychar(px):
-        while px.inputs.startswith(text, px.pos):
-            px.pos += clen
-        return True
-    return match_manychar
-
-
-def pOneManyChar(text):
-    clen = len(text)
-
-    def match_OneManychar(px):
-        if px.inputs.startswith(text, px.pos):
-            px.pos += clen
-            while px.inputs.startswith(text, px.pos):
-                px.pos += clen
-            return True
-        return False
-    return match_OneManychar
-
-
-def pOptionChar(text):
-    clen = len(text)
-
-    def match_optionchar(px):
-        if px.inputs.startswith(text, px.pos):
-            px.pos += clen
-        return True
-    return match_optionchar
-
-
-def pNotRange(chars, ranges):
-    pf = pAndRange(chars, ranges)
-
-    def match_ranges(px):
-        return not pf(px)
-    return match_ranges
-
-
-def pManyRange(chars, ranges):
-    pf = pAndRange(chars, ranges)
-
-    def match_manybitset(px):
-        while pf(px):
-            px.pos += 1
-        return True
-    return match_manybitset
-
-
-def pOneManyRange(chars, ranges):
-    pf = pAndRange(chars, ranges)
-
-    def match_onemanybitset(px):
-        c = 0
-        while pf(px):
-            px.pos += 1
-            c += 1
-        return c > 0
-    return match_onemanybitset
-
-
-def pOptionRange(chars, ranges):
-    pf = pAndRange(chars, ranges)
-
-    def match_optionbitset(px):
-        if pf(px):
-            px.pos += 1
-        return True
-    return match_optionbitset
+    return match
 
 # generate
 
 
-# PContext
-
-
-class PContext:
-    __slots__ = ['inputs', 'pos', 'epos',
-                 'headpos', 'ptree', 'state', 'memo', 'dic']
-
-    def __init__(self, inputs, spos, epos):
-        self.inputs = inputs
-        self.pos = spos
-        self.epos = epos
-        self.headpos = spos
-        self.ptree = None
-        self.state = None
-        self.memo = [PMemo() for x in range(1789)]
-        self.dic = {}
-
-# ParseTree
-
-
-def rowcol(urn, inputs, spos):
-    inputs = inputs[:spos + (1 if len(inputs) > spos else 0)]
-    rows = inputs.split(b'\n' if isinstance(inputs, bytes) else '\n')
-    return urn, spos, len(rows), len(rows[-1])-1
-
-
-def nop(s): return s
-
-
-UNKNOWN_SOURCE = '(unknown source)'
-
-
-class ParseTree(list):
-    def __init__(self, tag, inputs, spos=0, epos=None, urn=UNKNOWN_SOURCE):
-        self.tag_ = tag
-        self.inputs_ = inputs
-        self.spos_ = spos
-        self.epos_ = epos if epos is not None else len(inputs)
-        self.urn_ = urn
-
-    def getTag(self):
-        return self.tag_
-
-    def __eq__(self, tag):
-        return self.tag_ == tag
-
-    def getPosition(self):
-        return rowcol(self.urn_, self.inputs_, self.spos_)
-
-    def getEndPosition(self):
-        return rowcol(self.urn_, self.inputs_, self.epos_)
-
-    def decode(self):
-        inputs, spos, epos = self.inputs_, self.spos_, self.epos_
-        LF = b'\n' if isinstance(inputs, bytes) else '\n'
-        rows = inputs[:spos + (1 if len(inputs) > spos else 0)]
-        rows = rows.split(LF)
-        linenum, column = len(rows), len(rows[-1])-1
-        begin = inputs.rfind(LF, 0, spos) + 1
-        # print('@', spos, begin, inputs)
-        end = inputs.find(LF, spos)
-        # print('@', spos, begin, inputs)
-        if end == -1:
-            end = len(inputs)
-        # print('@[', begin, spos, end, ']', epos)
-        line = inputs[begin:end]  # .replace('\t', '   ')
-        mark = []
-        endcolumn = column + (epos - spos)
-        for i, c in enumerate(line):
-            if column <= i and i <= endcolumn:
-                mark.append('^' if ord(c) < 256 else '^^')
-            else:
-                mark.append(' ' if ord(c) < 256 else '  ')
-        mark = ''.join(mark)
-        return (self.urn_, spos, linenum, column, line, mark)
-
-    def message(self, msg='Syntax Error'):
-        urn, pos, linenum, cols, line, mark = self.decode()
-        return '{} ({}:{}:{}+{})\n{}\n{}'.format(msg, urn, linenum, cols, pos, line, mark)
-
-    def __eq__(self, tag):
-        return self.tag_ == tag
-
-    def isSyntaxError(self):
-        return self.tag_ == 'err'
-
-    def keys(self):
-        ks = []
-        for key in self.__dict__:
-            v = self.__dict__[key]
-            if isinstance(v, ParseTree):
-                ks.append(v)
-        return ks
-
-    def subs(self):
-        es = []
-        for i, child in enumerate(self):
-            es.append((child.spos_, '', child))
-        for key in self.__dict__:
-            v = self.__dict__[key]
-            if isinstance(v, ParseTree):
-                es.append((v.spos_, key, v))
-        es.sort()
-        return [(x[1], x[2]) for x in es]
-
-    def isEmpty(self):
-        return self.tag_ == 'empty'
-
-    def newEmpty(self):
-        return ParseTree('empty', self.inputs_, self.epos_, self.epos_, self.urn_)
-
-    def getNodeSize(self):
-        return len(self)
-
-    def getSubNodes(self):
-        return list(self)
-
-    def has(self, key):
-        if isinstance(key, str):
-            return hasattr(self, key) and isinstance(getattr(self, key), ParseTree)
-        if isinstance(key, int):
-            return key < self.getNodeSize()
-        return False
-
-    def get(self, key):
-        if not self.has(key):
-            return self.newEmpty()
-        if isinstance(key, str):
-            return getattr(self, key)
-        return self[key]
-
-    def set(self, key, t):
-        assert isinstance(t, ParseTree)
-        if key == '':
-            self.append(t)
-        else:
-            setattr(self, key, t)
-
-    def getToken(self, key=None, default_token=''):
-        if key is None:
-            s = self.inputs_[self.spos_:self.epos_]
-            return s.decode('utf-8') if isinstance(s, bytes) else s
-        return self.get(key).getToken() if self.has(key) else default_token
-
-    def substring(self, start=None, end=None):
-        if start is None:
-            if end is None:
-                return self.getToken()
-            s = self.inputs_[end.epos_:self.epos_]
-        else:
-            if end is None:
-                s = self.inputs_[self.spos_: start.spos_]
-            else:
-                s = self.inputs_[start.epos_:end.spos_]
-        return s.decode('utf-8') if isinstance(s, bytes) else s
-
-    def __str__(self):
-        s = self.inputs_[self.spos_:self.epos_]
-        return s.decode('utf-8') if isinstance(s, bytes) else s
-
-    def __repr__(self):
-        if self.isSyntaxError():
-            return self.message('Syntax Error')
-        sb = []
-        self.strOut(sb, indent='', tab='')
-        return "".join(sb)
-
-    def dump(self, indent='\n', tab='  ', tag=nop, edge=nop, token=nop):
-        if self.isSyntaxError():
-            print(self.message('Syntax Error'))
-        else:
-            sb = []
-            self.strOut(sb, indent, tab, '', tag, edge, token)
-            print("".join(sb))
-
-    def strOut(self, sb, indent='\n  ', tab='  ', prefix='', tag=nop, edge=nop, token=nop):
-        sb.append(indent + prefix + "[" + tag(f'#{self.getTag()} '))
-        subs = self.subs()
-        if len(subs) > 0:
-            next_indent = indent + tab
-            for label, child in subs:
-                prefix = edge(label) + ': ' if label != '' else ''
-                child.strOut(sb, next_indent, tab, prefix, tag, edge, token)
-            sb.append(indent + "]")
-        else:
-            sb.append(token(repr(str(self))))
-            sb.append("]")
-
-
 def PTreeConv(pt: PTree, urn, inputs):
     if pt.prev != None:
         ct = pt
@@ -1017,6 +838,10 @@ def PTreeConvNode(tag, urn, inputs, spos, epos, subnode):
     return t
 
 
+def pRule(peg, name, pf):
+    peg[name] = pf
+
+
 def generate(pf):
     # pf = self.generated[start.uname()]
     def parse(inputs, urn='(unknown source)', pos=0, epos=None, conv=PTreeConv):
